function [calcLRCIR,oneColQ,f,s] = initialComp(N,opt,restr)
% This function: computes a flag to indicate whether long-run cumulative IR 
% (LRCIR) should be computed; checks that the order of variables satisfies 
% Definition 3; checks that the SVAR is, in fact, set-identified using 
% condition 4.10; checks whether the restrictions apply to only one column 
% of Q (so that the approach of GMM18 is applicable); and does some 
% computations related to the convexity of the identified set.
% Inputs:
% - N: number of variables in VAR
% - opt: structure containing model information and options
% - restr: structure containing matrices representing restrictions

eqRestr = restr.eqRestr;
signRestr = restr.signRestr;
jshock = opt.jshock;
whichAlgo = opt.whichAlgo;

mZ = size(eqRestr,1); % No. of equality restrictions
mS = size(signRestr,1); % No. of sign restrictions (excluding normalisation)

% Generate flag to indicate whether LRCIR should be computed.
calcLRCIR = 0;

if mZ > 0
    calcLRCIR = any(eqRestr(:,4) == 4); % Flag to compute LRCIR
end

%% Check that order of variables satisfies Definition 3:
% Definition 3: Order the variables in the SVAR so that the number of zero
% restrictions f_i imposed on the ith column of Q (i.e., the rows of
% F_i(phi)) satisfy f_1 >= f_2 >= ... >= f_N >= 0. In case of ties, if the
% impulse response of interest is that to the j*-th structural shock, order
% the j*-th variable first. That is, set j* = 1 when no other column vector
% has a larger number of restrictions than q_j*. If j* >= 2, then order the
% variables so that f_{j*-1} > f_j*.

f = zeros(N,1);

if mZ > 0 % If there are equality restrictions

    for ii = 1:N % For each column of Q

        % Find number of equality restrictions on ith column of Q.
        f(ii) = sum((eqRestr(:,1)==ii & ...
        (eqRestr(:,4)==1 | eqRestr(:,4)==3)) | ...
        (eqRestr(:,2)==ii & (eqRestr(:,4)==2 | eqRestr(:,4)==4))); 

    end

end
    
if any(diff(flipud(f)) < 0)
    
    % If f_i < f_{i+1} for some i = 1,...,N-1, then the order of the
    % variables does not satisfy f_1 >= f_2 >= ... >= f_N.
    fprintf('\nOrder of variables does not satisfy f_1 >= f_2 >= ... >= f_N >= 0.\n');
    error('Error: Order of variables violates Definition 3!');

elseif find(f == f(jshock),1) ~= jshock

    % Check if the smallest value of i such that f_i = f_j* does not equal
    % j*, in which case the order of the variables does not satisfy
    % Definition 3.
    fprintf('\nj*-th variable is not ordered first among ties.\n');
    error('Error: Order of variables violates Definition 3!');

else

    fprintf('\nOrder of variables satisfies Definition 3.\n');

end

%% Check that SVAR is, in fact, set-identified using condition 4.10.
% f_i <= N-i for i = 1,...,N with strict inequality for at least one i.
if ~(all(f <= N - (1:N)') && any(f < N - (1:N)'))

    fprintf('\nEquality restrictions do not satisfy f_i <= N-i for i = 1,...,N, with strict inequality for at least one i.\n');
    error('Error: SVAR is not set-identified according to condition 4.10!.');

end

%% Check whether restrictions relate to only one column of Q.
% This is used to determine the applicability of the GMM18 algorithm for
% computing the bounds of the identified set.
s = zeros(N,1);

if mS > 0 % If there are sign restrictions
       
    for ii = 1:N % For each column of Q.
        
        % Count number of inequality restrictions on ith column of Q
        s(ii) = sum((signRestr(:,2)==ii & signRestr(:,5)==1) | ...
            (signRestr(:,1)==ii & signRestr(:,5)==2));

    end
    
end

totRestr = f + s; % Total number of restrictions on each column of Q
    
% Construct indicator for restrictions on one column of Q only.
oneColQ = (sum(totRestr > 0) <= 1); 

if oneColQ == 1 && whichAlgo == 3

    fprintf('\nRestrictions apply to one column of Q.'); 
    fprintf('\nComputing bounds of identified set using approach of Gafarov, Meier and Montiel-Olea (2018)...\n');
    
elseif whichAlgo == 3 && oneColQ~=1
    
    fprintf('\nRestrictions apply to multiple columns of Q.\n');
    error('Error: Approach of Gafarov, Meier and Montiel-Olea (2018) not applicable! Choose a different algorithm.');
    
elseif whichAlgo == 1
    
    fprintf('\nComputing bounds of identified set using Algorithm 1...\n');
    
elseif whichAlgo == 2
    
    fprintf('\nComputing bounds of identified set using Algorithm 2...\n');
    
end

%% Assess convexity of identified set using Proposition B.1.

if jshock == 1 && (f(1) < N-1) 
 
    if mS == 0
        
        % If j* = 1, f_1 < N-1 and there are no sign restrictions, then the
        % identified set is convex for almost every phi by 
        % Proposition B.1(I)(i).
        fprintf('\nIdentified set is convex based on Proposition B.1(I)(i).\n');
        
    elseif s(1) > 0 && all(s(2:end) == 0)
        
        % If equality restrictions satisfy the conditions in
        % Proposition B.1(I)(i) and there are sign restrictions
        % constraining q_1 only, then the identified set is convex
        % conditional on being nonempty.
        fprintf('\nIdentified set is convex (conditional on being nonempty) based on Proposition B.1(II)(iv)');
            
    end

elseif jshock >= 2 && all(f(1:jshock-1) < N-(1:(jshock-1))') 
        
    if mS == 0
    
        % If j* >= 2, f_i < N-i for all i = 1,...,j*-1 and there are no 
        % sign restrictions, then the identified set is convex for almost 
        % every phi by Proposition B.1(I)(ii).
        fprintf('\nIdentified set is convex based on Proposition B.1(I)(ii).\n');
        
        
    elseif s(jshock) > 0 &&  all(s(setdiff(1:N,jshock))==0)
        
        % If equality restrictions satisfy the conditions in
        % Proposition B.1(I)(ii) and there are sign restrictions
        % constraining q_j* only, then the identified set is convex
        % conditional on being nonempty.
        fprintf('\nIdentified set is convex (conditional on being nonempty) based on Proposition B.1(II)(iv).\n');
        
    end
    
elseif jshock >= 2 
    
    % Check if there exists 1 <=  i* <= j*-1 such that f_i < n-i for all
    % i = i*+1,...,j* and the necessary condition for exact identification 
    % of [q_1,...,q_i*] is satisfied (f_i = n-i for all i = 1,...,i*).
    
    istar = find(f(1:(jshock-1)) == N-(1:(jshock-1))',1,last);
    flag = all(f((istar+1):jshock) < N-((istar+1):jshock)');
    
    if flag == 1 && mS == 0
    
        % If j* >= 2, there exists 1 <=  i* <= j*-1 such that f_i < n-i 
        % for all i = i*+1,...,j* and [q_1,...,q_i*] is exactly identified,
        % and there are no sign restrictions, then the identified set is
        % convex for almost every phi by Proposition B.1(I)(iii).
        convInfo.propCase = 3;
        fprintf('\nIdentified set is convex based on Proposition B.1(I)(iii).\n');
        
    elseif flag == 1 && s(jshock) > 0 &&  all(s(setdiff(1:N,jshock))==0)
        
        % If j* >= 2, there exists 1 <=  i* <= j*-1 such that f_i < n-i 
        % for all i = i*+1,...,j* [q_1,...,q_i*] is exactly identified,
        % and there sign restrictions constraining q_j* only, then the 
        % identified set is convex conditional on being nonempty.
        fprintf('\nIdentified set is convex (conditional on being nonempty) based on Proposition B.1(II)(v).\n');     
        
    else
        
        fprintf('\nConditions for Proposition B.1 are not satisfied. Identified set may be nonconvex.\n');
        
    end
    
end

end