%% Construct data for estimating VAR and compute MLE.
YY = data(opt.p+1:end,:); % LHS variables
XX = lagmatrix(data,1:opt.p); % Matrix of regressors
XX = XX(opt.p+1:end,:); % Drop initial missing observations

if opt.const == 1 % Add constant to matrix of regressors if required
    XX = [ones(length(XX),1) XX];   
end

N = size(YY,2); % Number of variables
Ni = length(opt.ivar); % Number of variables of interest
M = N*opt.p + opt.const; % Number of parameters in each equation
T = length(YY); % Number of observations used in estimating VAR

% Compute useful variables and check whether analytical approach of GMM18 
% is applicable.
[opt.calcLRCIR,oneColQ,restr.f,restr.s] = initialComp(N,opt,restr);

%% Conduct posterior inference on IRs.
% j-th column of B gives MLE/OLS coefficients of j-th reduced form VAR 
% equation. If constant included, first row corresponds to constant term.
phiHat.B = (XX'*XX)\XX'*YY;
phiHat.S = (YY - XX*phiHat.B)'*(YY-XX*phiHat.B);
phiHat.Sigma = (1/T)*phiHat.S; % Variance matrix of VAR innovations
phiHat.P = (XX'*XX)\eye(M);
phiHat.cholP = chol(phiHat.P,'lower');

% Storage arrays.
rMinPost = zeros(opt.Kdraws,opt.H+1,Ni);
rMaxPost = rMinPost;
rSinglePriorPost = rMinPost;
B = zeros(N*M,opt.Kdraws);
Sigma = zeros(N,N,opt.Kdraws);
convInfo.convInd = zeros(opt.Kdraws,1);

phiDraw = 0; % Counter for no. of draws from posterior of phi
nonStableCount = 0; % Counter for no. of draws with nonstable VAR
nonEmptyDraw = 0; % Counter for no. of draws with non-empty IS

while nonEmptyDraw < opt.Kdraws

    % Posterior sampler with noninformative improper (Jeffreys) prior.
    % See the file 'niwPrior.m' for details about implementing a
    % normal-inverse-Wishart prior.
    phi.Sigma = iwishrnd(phiHat.S,T-M);
    phi.Sigmatr = chol(phi.Sigma,'lower');
    phi.Sigmatrinv = phi.Sigmatr\eye(N);
    phi.B = phiHat.B(:) + kron(phi.Sigmatr,phiHat.cholP)*randn(M*N,1);
    phiDraw = phiDraw + 1;

    % Generate coefficients in reduced-form VMA representation and, if 
    % required, long-run (cumulative) IR. Also generate versions of these 
    % objects post-multiplied by Sigmatr, and check invertibility of the 
    % VAR representation.
    [irs,nonStable] = genVMA(phi,opt);
    nonStableCount = nonStableCount + nonStable;
    
    % If VAR is stable, proceed. Otherwise, redraw phi.
    
    if nonStable == 0
        
        phiDraw = phiDraw + 1;
        
    if opt.whichAlgo == 3 % If using approach from GMM18
        
        % Use analytical approach to determine whether IS is nonempty.
        Qempty(phiDraw) = checkEmptyISAnalytical(restr,irs,phi,opt);
        
        if Qempty(phiDraw) == 0 % If IS is nonempty
            
            % Set maximum number of draws of Q to a large number.
            opt.maxDraw = 1e6;
            % Use Algorithm 1 (Step 2) to draw Q0 from space of orthonormal
            % matrices satisfying equality and sign restrictions. This 
            % draw is used to compute the IR under a single prior for Q.
            [Q0,restr.F,restr.S,~] = drawQ(restr,irs,phi,opt);
        
        end
        
    else % If using Algorithm 1 or 2
        
        % Use Algorithm 1 (Step 2) to determine whether IS is nonempty and 
        % draw Q0 from space of orthonormal matrices satisfying equality 
        % and sign restrictions. This draw is used to compute the IR under 
        % a single prior for Q. 
        [Q0,restr.F,restr.S,Qempty(phiDraw)] = ...
            drawQ(restr,irs,phi,opt);
        
    end
               
    if Qempty(phiDraw) == 0 % If IS non-empty

            % Store reduced-form parameters
            nonEmptyDraw = nonEmptyDraw + 1;
            Sigma(:,:,nonEmptyDraw) = phi.Sigma;
            B(:,nonEmptyDraw) = phi.B;

            % Compute draw from posterior of IRs given single prior for Q.

            for hh = 1:opt.H+1 % For each horizon

                for ii = 1:Ni % For each variable of interest

                    rSinglePriorPost(nonEmptyDraw,hh,ii) = ...
                        irs.vmaGK(opt.ivar(ii),:,hh)*Q0(:,opt.jshock);

                end

            end
            
           % Find upper and lower bounds of IR IS.
            
           if opt.whichAlgo == 1 % Use Algorithm 1               
               
               % Users without the Parallel Computing Toolbox can replace 
               % the function numericalBoundsPar with numericalBounds.
               [rMinPost(nonEmptyDraw,:,:),rMaxPost(nonEmptyDraw,:,:)] = ...
                   numericalBoundsPar(restr,irs,phi,opt,optimOptions);
               
           elseif opt.whichAlgo == 2 % Use Algorithm 2
                     
               [rMinPost(nonEmptyDraw,:,:),rMaxPost(nonEmptyDraw,:,:)] = ...
                   approximateBounds(restr,irs,phi,opt);
                             
           elseif opt.whichAlgo == 3 % Use results from GMM18
               
               [rMinPost(nonEmptyDraw,:,:),rMaxPost(nonEmptyDraw,:,:)] = ...
                   analyticalBounds(restr,irs,phi,opt);

           end
           
           if mod(opt.Kdraws-nonEmptyDraw,opt.dispIter) == 0
              
               fprintf('\n%d draws with non-empty identified set remaining...',...
               opt.Kdraws-nonEmptyDraw);
               
           end     

    end
    
    end
      
end

% Compute posterior mean under single prior.
rSinglePriorMean = permute(mean(rSinglePriorPost,1),[2 3 1]);

% Compute posterior mean bounds.
meanlb = permute(mean(rMinPost,1),[2 3 1]);
meanub = permute(mean(rMaxPost,1),[2 3 1]);

% Compute robustified credible region for IS.
[credlb,credub] = credibleRegion(rMinPost,rMaxPost,opt);
% Compute highest posterior density (HPD) interval under single prior.
[hpdlb,hpdub] = highestPosteriorDensity(rSinglePriorPost,opt);

% Compute posterior plausibility of the identifying restrictions.
postPlaus = (1 - sum(Qempty)/length(Qempty));
fprintf('\nPosterior plausibility of identifying restrictions: %0.3f.\n',...
    postPlaus)

postMeanBoundWidth = meanub - meanlb; % Width of posterior mean bounds.
hpdWidth = hpdub - hpdlb; % Width of highest posterior density regions
credWidth = credub - credlb; % Width of robustified credible region

priorInformativeness = 1 - hpdWidth./credWidth; % Informativeness of prior

% Proportion of draws where VAR is noninvertible.
nonInvertPc = nonStableCount/phiDraw*100;
fprintf('\nVAR representation noninvertible in %0.3f per cent of draws. \n',...
    nonInvertPc);

%% Plot IRs
for ii = 1:Ni
    
    figure;
    h1 = plot(0:opt.H,meanlb(:,ii),'color','k','LineStyle','--');
    hold on;
    plot(0:opt.H,meanub(:,ii),'color','k','LineStyle','--');
    h2 = plot(0:opt.H,credlb(:,ii),'-k','LineWidth',2);
    plot(0:opt.H,credub(:,ii),'-k','LineWidth',2);
    h3 = plot(0:opt.H,rSinglePriorMean(:,ii),'ko','MarkerFaceColor','k');
    h4 = plot(0:opt.H,hpdlb(:,ii),'--k','LineWidth',2);
    plot(0:opt.H,hpdub(:,ii),'--k','LineWidth',2);
    
    if irfUnits(opt.ivar(ii)) == 0
        ylabel('%');
    elseif irfUnits(opt.ivar(ii)) == 1
        ylabel('ppt');
    end
    
    title(sprintf('Response of %s to Shock in %s',...
        varnames{opt.ivar(ii)},varnames{opt.jshock}));
    xlabel('Horizon');
    
    for hh = 0:opt.H
        line([hh hh],[meanlb(hh+1,ii) meanub(hh+1,ii)],'color','k');
    end

    line([0 opt.H],[0 0],'color','black','LineStyle',':');
    
    legend([h1 h2 h3 h4],{'Posterior mean bounds',...
        'Bounds of credible region','Posterior mean (single prior)',...
        'HPD region (single prior)'},'Location','Best');
    
end