% Conduct robust Bayesian inference about impulse responses (IRs) in an 
% SVAR set-identified with equality and/or sign restrictions. Application 
% is to the SVAR considered in Beaudry, Nam and Wang (2011), as described 
% in Arias, Rubio-Ramirez and Waggoner (2018).

clear variables
close all
clc

oldFolder = pwd;
cd ..
addpath([oldFolder,'/GKFunctions']);
cd(oldFolder);

%% Import data.
% These are obtained from the Matlab documentation from Arias,
% Rubio-Ramirez and Waggoner (2018), available on Jonas Arias' website:
% https://sites.google.com/site/jonasarias/
% Variables are:
% 1: Adjusted TFP (z)
% 2: Stock Prices (p)
% 3: Consumption (c)
% 4: Real Interest Rate (r)
% 5: Hours Worked (h)
dataRaw = importdata('data_BNW11.csv');

% Order columns of dataset to be consistent with Definition 3.
% The number of equality restrictions imposed on the ith column of Q (f_i) 
% must satisfy f_1 >= f_2 >= ... >= f_N >= 0. In the case of ties, if the 
% IR of interest is with respect to the jth structural shock, let the jth 
% variable be ordered first.
data = dataRaw.data(:,[4 3 5:end])*100;
% Stock prices need to be ordered first.

% Set variable names for plotting figures.
varnames = {'Stock Prices','Adjusted TFP','Consumption',...
    'Real Interest Rate','Hours Worked'};
% Set units for IR. 0 = per cent, 1 = percentage points.
irfUnits = [0 0 0 1 0];

%% Options.
opt.p = 4; % No. of lags in VAR
opt.const = 1; % const = 1 if constant in VAR, = 0 otherwise
opt.ivar = [1 2 3 4 5];  % Indices of variables whose IR is of interest
opt.cumIR = []; % Indices of variables for which cumulative IR should be used
opt.jshock = 1; % Index of structural shock of interest
opt.H = 40; % Terminal horizon for IR
opt.Kdraws = 10000; % No. of draws from posterior of phi with non-empty IS
opt.aalpha = 0.68; % Credibility level for robust credible and HPD intervals
opt.gridLength = 1000; % Number of points on discrete grid when computing intervals 
opt.signNorm = 0; % = 0 if sign normalisation imposed on A0, = 1 if on A0^(-1)
opt.maxDraw = 3000; % Maximum no. of draws from set of orthonormal matrices
opt.Qdraws = 5; % No. of initial values of Q in optimisation step of Algorithm 1
opt.KTilde = 10000; % No. of draws of Q used in Algorithm 2
opt.dispIter = 100; % Print number of draws remaining every dispIter draws

% Choose algorithm used to compute bounds of identified set at each draw 
% from posterior of phi.
% 1 = Algorithm 1 (optimisation)
% 2 = Algorithm 2 (simulation)
% 3 = Analytical approach from GMM18
opt.whichAlgo = 3;

% Options for optimisation step in Algorithm 1.
optimOptions = optimoptions('fmincon');
optimOptions.Display = 'off'; % Do not display optimisation output
% Provide analytical gradient of objective function.
optimOptions.GradObj = 'on'; 
% Provide analytical gradient of nonlinear constraint.
optimOptions.SpecifyConstraintGradient = true; 
optimOptions.CheckGradients = false; % Check provided gradients numerically

rng(1); % Set seed for random number generator

%% Input equality and sign restrictions.
% Each row of eqRestr contains a quartet (i,j,l,t) representing a 
% particular equality restriction, where t is the type of restriction:
% t = 1: the (ij)th element of A0 is zero
% t = 2: the (ij)th element of A0^(-1) is zero
% t = 3: the (ij)th element of A_l is zero
% t = 4: the (ij)th element of the long-run cumulative IR (LRCIR) is zero 
% The value of l does not matter except when t = 3.
% restr.eqRestr = []; % No equality restrictions
restr.eqRestr = ...
    [2 1 0 2]; % (A0^(-1))_12 = 0 - response of TFP to 'optimism shock' is zero

% Each row of signRestr contains a quintet (i,j,h,s,t) representing a
% particular sign restriction, where t is the type of restriction:
% t = 1: the IR of the ith variable with respect to the jth shock at the 
% hth horizon is nonnegative (s = 1) or nonpositive (s = -1).
% t = 2: the (ij)th element of A0 is nonnegative (s = 1) or nonpositive 
% (s = -1). 
% Warning: signRestr should NOT include sign normalisations. These are
% controlled by the 'signNorm' option above. If the sign normalisation is
% placed on A0^(-1), do not place sign restrictions on the response of the
% i-th variable to the i-th shock at h = 0 for i = 1,...,N. Otherwise,
% constraints will be colinear.
% restr.signRestr = []; % No sign restrictions on IRs (sign normalisations only)
restr.signRestr = ...
    [1 1 0 1 1]; % Response of stock prices to stock price shock is nonnegative on impact
     
%% Conduct posterior inference and produce graphs of impulse responses.
mainfile;