% Conduct robust Bayesian inference about impulse responses (IRs) in an 
% SVAR set-identified with equality and/or sign restrictions. Application 
% is to the SVAR considered in Gafarov, Meier and Montiel-Olea (2018).

clear variables
close all
clc

oldFolder = pwd;
cd ..
addpath([oldFolder,'/GKFunctions']);
cd(oldFolder);

%% Import data.
% As described in Gafarov, Meier and Montiel-Olea (2018), these data were 
% obtained from Peter Karadi's personal website:
% https://sites.google.com/site/pkaradi696/research
% Variables are:
% 1: year
% 2: month
% 3: log CPI (cpi)
% 4: log Industrial Production (ip)
% 5: Federal funds rate (ffr)
% 6: 1-year Treasury Bond Rate (tb1)
% 7: 2-year Treasury Bond Rate (tb2)
dataRaw = importdata('data_GMM18.csv');

% Order columns of dataset to be consistent with Definition 3.
% The number of equality restrictions imposed on the ith column of Q (f_i) 
% must satisfy f_1 >= f_2 >= ... >= f_N >= 0. In the case of ties, if the 
% IR of interest is with respect to the jth structural shock, let the jth 
% variable be ordered first.
data = dataRaw.data(:,[7 3:5]); % (tb2 cpi ip ffr)
% Truncate sample period.
data = data(dataRaw.data(:,1)+(dataRaw.data(:,2)-1)/12 <= 2008+7/12,:);
data = diff(data); % Transform variables

% Set variable names for plotting figures.
varnames = {'2-year Treasury Yield','CPI','IP','Federal Funds Rate'};
% Set units for IR. 0 = per cent, 1 = percentage points.
irfUnits = [1 0 0 1];

%% Options.
opt.p = 12; % No. of lags in VAR
opt.const = 1; % const = 1 if constant in VAR, = 0 otherwise
opt.ivar = [1 2 3 4];  % Indices of variables whose IR is of interest
opt.cumIR = [1 2 3 4]; % Indices of variables for which cumulative IR should be used
opt.jshock = 1; % Index of structural shock of interest
opt.H = 36; % Terminal horizon for IR
opt.Kdraws = 1000; % No. of draws from posterior of phi with non-empty IS
opt.aalpha = 0.68; % Credibility level for robust credible and HPD intervals
opt.gridLength = 1000; % Number of points on discrete grid when computing intervals 
opt.signNorm = 0; % = 0 if sign normalisation imposed on A0, = 1 if on A0^(-1)
opt.maxDraw = 3000; % Maximum no. of draws from set of orthonormal matrices
opt.Qdraws = 5; % No. of initial values of Q in optimisation step of Algorithm 1
opt.KTilde = 10000; % No. of draws of Q used in Algorithm 2
opt.dispIter = 100; % Print number of draws remaining every dispIter draws

% Choose algorithm used to compute bounds of IS at each draw from the
% posterior of phi.
% 1 = Algorithm 1 (optimisation)
% 2 = Algorithm 2 (simulation)
% 3 = Analytical approach from GMM18
opt.whichAlgo = 3;

% Options for optimisation step in Algorithm 1.
optimOptions = optimoptions('fmincon');
optimOptions.Display = 'off'; % Do not display optimisation output
% Provide analytical gradient of objective function.
optimOptions.GradObj = 'on'; 
% Provide analytical gradient of nonlinear constraint.
optimOptions.SpecifyConstraintGradient = true; 
optimOptions.CheckGradients = false; % Check provided gradients numerically

rng(1); % Set seed for random number generator

%% Input equality and sign restrictions.
% Each row of eqRestr contains a quartet (i,j,l,t) representing a 
% particular equality restriction, where t is the type of restriction:
% t = 1: the (ij)th element of A0 is zero
% t = 2: the (ij)th element of A0^(-1) is zero
% t = 3: the (ij)th element of A_l is zero
% t = 4: the (ij)th element of the long-run cumulative IR (LRCIR) is zero 
% The value of l does not matter except when t = 3.
restr.eqRestr = ...
    [4 1 0 2]; % {A0^(-1)}_{41} = 0 - ffr does not respond on impact to UMP shock

% Each row of signRestr contains a quintet (i,j,h,s,t) representing a
% particular sign restriction, where t is the type of restriction:
% t = 1: the IR of the ith variable with respect to the jth shock at the 
% hth horizon is nonnegative (s = 1) or nonpositive (s = -1).
% t = 2: the (ij)th element of A0 is nonnegative (s = 1) or nonpositive 
% (s = -1). 
% Warning: signRestr should NOT include sign normalisations. These are
% controlled by the 'signNorm' option above. If the sign normalisation is
% placed on A0^(-1), do not place sign restrictions on the response of the
% i-th variable to the i-th shock at h = 0 for i = 1,...,N. Otherwise,
% constraints will be colinear.
restr.signRestr = ...
    [2 1 0 -1 1; % Response of CPI to UMP shock is negative
     3 1 0 -1 1; % Response of IP to UMP shock is negative
     1 1 0 1 1]; % Response of tb2 to UMP shock is positive

% Note that a positive UMP shock raises the 2-year Treasury yield.
% Multiply IRFs by -1 for direct comparison against results from Gafarov,
% Meier and Montiel-Olea (2018), who consider responses to a shock that
% lowers the 2-year Treasury yield.
 
%% Conduct posterior inference and produce graphs of impulse responses.
mainfile;