% This file replicates the results in Giacomini and Kitagawa (2020).

clear variables
close all
clc

oldFolder = pwd;
cd ..
addpath('GKFunctions');
cd(oldFolder);

%% Import data.
% Variables 1, 3, 4 and 5 are obtained from Granziera et al. (2018).
% Variable 2 is obtained from FRED (series A939RX0Q048SBEA).
% Variables are:
% 1. Real GDP per capita in deviations from a linear trend 
% 2. Real GDP per capita in log first differences (*100) (y)
% 3. Inflation (%, annualised) (pi)
% 4. Interest rate (%) (i)
% 5. Velocity of money (v)
dataRaw = importdata('datampshock.csv');

% Compute real money balances (m) from output (in deviations from linear 
% trend) and velocity of money. This follows Granziera et al. (2018).
dataRaw.data(:,5) = dataRaw.data(:,5)+dataRaw.data(:,1);

% Order columns of dataset to be consistent with Definition 3.
% The number of equality restrictions imposed on the ith column of Q (f_i) 
% must satisfy f_1 >= f_2 >= ... >= f_N >= 0. In the case of ties, if the 
% IR of interest is with respect to the jth structural shock, let the jth 
% variable be ordered first.
data = dataRaw.data(:,[4 2 3 5]); % (i, y, pi, m)

% Set variable names for plotting figures.
varnames = {'Interest rate','Real GDP Per Capita','Inflation',...
    'Real Money Balances'};

%% Options.
opt.p = 2; % No. of lags in VAR
opt.const = 1; % const = 1 if constant in VAR, = 0 otherwise
opt.ivar = [2];  % Indices of variables whose IR is of interest
opt.cumIR = [2]; % Indices of variables for which cumulative IR should be used
opt.jshock = 1; % Index of structural shock of interest
opt.H = 24; % Terminal horizon for IR
opt.Kdraws = 1000; % No. of draws from posterior of phi with non-empty IS
opt.aalpha = 0.9; % Credibility level for robust credible and HPD intervals
opt.gridLength = 1000; % Number of points on discrete grid when computing intervals 
opt.signNorm = 0; % = 0 if sign normalisation imposed on A0, = 1 if on A0^(-1)
opt.maxDraw = 3000; % Maximum no. of draws from set of orthonormal matrices
opt.Qdraws = 5; % No. of initial values of Q in optimisation step of Algorithm 1
opt.dispIter = 100; % Print number of draws remaining every dispIter draws

% Options for optimisation step in Algorithm 1.
optimOptions = optimoptions('fmincon');
optimOptions.Display = 'off'; % Do not display optimisation output
% Provide analytical gradient of objective function.
optimOptions.GradObj = 'on'; 
% Provide analytical gradient of nonlinear constraint.
optimOptions.SpecifyConstraintGradient = true; 
optimOptions.CheckGradients = false; % Check provided gradients numerically

rng(1); % Set seed for random number generator

%% Construct data for estimating VAR and compute MLE.
YY = data(opt.p+1:end,:); % LHS variables
XX = lagmatrix(data,1:opt.p); % Matrix of regressors
XX = XX(opt.p+1:end,:); % Drop initial missing observations

if opt.const == 1 % Add constant to matrix of regressors if required
    XX = [ones(length(XX),1) XX];   
end

N = size(YY,2); % Number of variables
Ni = length(opt.ivar); % Number of variables of interest
M = N*opt.p + opt.const; % Number of parameters in each equation
T = length(YY); % Number of observations used in estimating VAR

% j-th column of B gives MLE/OLS coefficients of j-th reduced form VAR 
% equation. If constant included, first row corresponds to constant term.
phiHat.B = (XX'*XX)\XX'*YY;
phiHat.S = (YY - XX*phiHat.B)'*(YY-XX*phiHat.B);
phiHat.Sigma = (1/T)*phiHat.S; % Variance matrix of VAR innovations
phiHat.P = (XX'*XX)\eye(M);
phiHat.cholP = chol(phiHat.P,'lower');

% The following describes how to impose the equality and sign restrictions 
% in each model.

% Each row of eqRestr contains a quartet (i,j,l,t) representing a 
% particular equality restriction, where t is the type of restriction:
% t = 1: the (ij)th element of A0 is zero
% t = 2: the (ij)th element of A0^(-1) is zero
% t = 3: the (ij)th element of A_l is zero
% t = 4: the (ij)th element of the long-run cumulative IR (LRCIR) is zero 
% The value of l does not matter except when t = 3.

% Each row of signRestr contains a quintet (i,j,h,s,t) representing a
% particular sign restriction, where t is the type of restriction:
% t = 1: the IR of the ith variable with respect to the jth shock at the 
% hth horizon is nonnegative (s = 1) or nonpositive (s = -1).
% t = 2: the (ij)th element of A0 is nonnegative (s = 1) or nonpositive 
% (s = -1). 
% Warning: signRestr should NOT include sign normalisations. These are
% controlled by the 'signNorm' option above. If the sign normalisation is
% placed on A0^(-1), do not place sign restrictions on the response of the
% i-th variable to the i-th shock at h = 0 for i = 1,...,N. Otherwise,
% constraints will be colinear.

%% Model 0.
restr.eqRestr = []; % No equality restrictions
restr.signRestr = []; % No sign restrictions on IRFs (sign normalisations only)
modelStr = 'Model0';
computeIR = 0; % Do not compute informativeness of restrictions
mainfile_GK;
postMeanBoundWidthGKModel0 = postMeanBoundWidthGK;
postMeanBoundWidthGMMModel0 = postMeanBoundWidthGMM;

%% Model 1.
restr.signRestr = [1 1 0 1 1; % Response of i to i shock is nonnegative on impact
             1 1 1 1 1; % As above but after one quarter
             3 1 0 -1 1; % Response of pi to i shock is nonpositive on impact
             3 1 1 -1 1; % As above but after one quarter
             4 1 0 -1 1; % Response of m to i shock is nonpositive on impact
             4 1 1 -1 1]; % As above but after one quarter
modelStr = 'Model1';
computeIR = 1; % Compute informativeness of restrictions
mainfile_GK;

%% Model 2.
restr.eqRestr = [1 2 0 1]; % A_12 = 0
modelStr = 'Model2';
mainfile_GK;

%% Model 3.
restr.eqRestr = [2 1 0 2]; % (A0^(-1))_21 = 0
modelStr = 'Model3';
mainfile_GK;

%% Model 4.
restr.eqRestr = [2 1 0 4]; % LRCIR(y,i) = 0
modelStr = 'Model4';
mainfile_GK;

%% Model 5.
restr.eqRestr = [1 2 0 1; % A0_12 = 0
           2 1 0 2]; % (A0^(-1))_21 = 0 (i.e. IR_0(y,i) = 0)
modelStr = 'Model5';
mainfile_GK;

%% Model 6.
restr.eqRestr = [1 2 0 1; % A0_12 = 0
           2 1 0 4]; % LRCIR(y,i) = 0
modelStr = 'Model6';
mainfile_GK;       

%% Model 7.
restr.eqRestr = [2 1 0 2; % (A0^(-1))_12 = 0
           2 1 0 4]; % LRCIR(y,i) = 0
modelStr = 'Model7';
mainfile_GK;          
       
% Generate graphs and tables for paper.
replicateGKGraphs;
replicateGKTable;